# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/025_CLI_Testing.ipynb.

# %% auto 0
__all__ = ['logger', 'testing_install_deps']

# %% ../nbs/025_CLI_Testing.ipynb 1
from typing import *

import typer

from ._components.logger import get_logger
from ._components.test_dependencies import _install_testing_deps

# %% ../nbs/025_CLI_Testing.ipynb 5
logger = get_logger(__name__)

# %% ../nbs/025_CLI_Testing.ipynb 8
_testing_app = typer.Typer(help="Commands for managing fastkafka testing")

# %% ../nbs/025_CLI_Testing.ipynb 9
@_testing_app.command(
    "install_deps",
    help="Installs dependencies for FastKafka app testing",
)
def testing_install_deps() -> None:
    try:
        _install_testing_deps()
    except Exception as e:
        typer.secho(f"Unexpected internal error: {e}", err=True, fg=typer.colors.RED)
        raise typer.Exit(1)
